@@ -976,6 +976,8 @@ protected:
 		return( NULL );
 	}
 
+	void *					Get_Array		(void) const { return( m_Stack.Get_Entry(1) ); }
+
 
 private:
 
@@ -1136,6 +1138,7 @@ public:
 	virtual bool					Close				(void);
 
 	virtual const CSG_String &		Get_File_Name		(void)	const	{	return( m_FileName );	}
+	virtual CSG_String				Get_File_Name		(size_t Index)	{	return( m_FileName );	}
 	virtual TSG_File_Type			Get_File_Type		(void)	const	{	return( SG_FILE_TYPE_NORMAL );	}
 
 	class wxStreamBase *			Get_Stream			(void)	const	{	return( (class wxStreamBase *)m_pStream );	}
@@ -1645,6 +1648,9 @@ typedef enum
 
 	CALLBACK_DATAOBJECT_COLORS_GET,
 	CALLBACK_DATAOBJECT_COLORS_SET,
+
+	CALLBACK_DATAOBJECT_CLASSIFY,
+
 	CALLBACK_DATAOBJECT_PARAMS_GET,
 	CALLBACK_DATAOBJECT_PARAMS_SET,
 
@@ -1747,6 +1753,9 @@ SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_asImage	(class CSG_Data_Object *pD
 
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Colors_Get	(class CSG_Data_Object *pDataObject, class CSG_Colors *pColors);
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Colors_Set	(class CSG_Data_Object *pDataObject, class CSG_Colors *pColors);
+
+SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Classify	(class CSG_Data_Object *pDataObject, const class CSG_MetaData &Options);
+
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Params_Get	(class CSG_Data_Object *pDataObject, class CSG_Parameters *pParameters);
 SAGA_API_DLL_EXPORT bool					SG_UI_DataObject_Params_Set	(class CSG_Data_Object *pDataObject, class CSG_Parameters *pParameters);
 
