@@ -97,15 +97,9 @@ CLandsat_Import::CLandsat_Import(void)
 		), 0
 	);
 
-	Parameters.Add_Choice(Parameters("PROJECTION"),
+	Parameters.Add_Grid_Resampling("PROJECTION",
 		"RESAMPLING", _TL("Resampling"),
-		_TL(""),
-		CSG_String::Format("%s|%s|%s|%s",
-			_TL("Nearest Neighbour"),
-			_TL("Bilinear Interpolation"),
-			_TL("Bicubic Spline Interpolation"),
-			_TL("B-Spline Interpolation")
-		), 3
+		_TL("")
 	);
 
 	Parameters.Add_Bool("",
