@@ -61,11 +61,12 @@
 class C3D_Viewer_PointCloud_Panel : public CSG_3DView_Panel
 {
 public:
-	C3D_Viewer_PointCloud_Panel(wxWindow *pParent, CSG_PointCloud *pPoints, int Coloring, int Attribute);
+	C3D_Viewer_PointCloud_Panel(wxWindow *pParent, CSG_PointCloud *pPoints, int Coloring, int Attribute, const CSG_Rect &Extent);
 
 	static CSG_String			Get_Usage				(void);
 
 	void						Set_Extent				(CSG_Rect Extent);
+	const CSG_Rect &			Get_Extent				(void) const { return( m_Extent ); }
 
 
 protected:
@@ -122,7 +123,7 @@ END_EVENT_TABLE()
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-C3D_Viewer_PointCloud_Panel::C3D_Viewer_PointCloud_Panel(wxWindow *pParent, CSG_PointCloud *pPoints, int Coloring, int Attribute)
+C3D_Viewer_PointCloud_Panel::C3D_Viewer_PointCloud_Panel(wxWindow *pParent, CSG_PointCloud *pPoints, int Coloring, int Attribute, const CSG_Rect &Extent)
 	: CSG_3DView_Panel(pParent)
 {
 	m_pPoints = pPoints;
@@ -136,23 +137,27 @@ C3D_Viewer_PointCloud_Panel::C3D_Viewer_PointCloud_Panel(wxWindow *pParent, CSG_
 	}
 
 	//----------------------------------------------------
-	m_Parameters.Add_Double    ("GENERAL"    , "DETAIL"       , _TL("Level of Detail"   ), _TL(""), 100., 0., true, 100., true);
+	m_Parameters.Add_Double    ("GENERAL"       , "DETAIL"        , _TL("Level of Detail"   ), _TL(""), 100., 0., true, 100., true);
 
-	m_Parameters.Add_Choice    ("GENERAL"    , "COLORING"     , _TL("Coloring"          ), _TL(""), CSG_String::Format("%s|%s|%s|%s", _TL("Classified"), _TL("Discrete Colors"), _TL("Graduated Colors"), _TL("RGB Coded Values")), Coloring);
-	m_Parameters.Add_Choice    ("COLORING"   , "COLORS_ATTR"  , _TL("Attribute"         ), _TL(""), Attributes, Attribute);
-	m_Parameters.Add_FixedTable("COLORING"   , "COLORS_LUT"   , _TL("Lookup Table"      ), _TL(""));
-	m_Parameters.Add_Colors    ("COLORING"   , "COLORS"       , _TL("Colors"            ), _TL(""));
-	m_Parameters.Add_Choice    ("COLORING"   , "COLORS_FIT"   , _TL("Color Fit"         ), _TL(""), CSG_String::Format("%s|%s", _TL("to extent when zoomed"), _TL("constant value range")), 0);
-	m_Parameters.Add_Range     ("COLORS_FIT" , "COLORS_RANGE" , _TL("Value Range"       ), _TL(""));
-	m_Parameters.Add_Double    ("COLORS_FIT" , "COLORS_STDDEV", _TL("Standard Deviation"), _TL(""), 1.5, 0.1, true);
+	m_Parameters.Add_Choice    ("GENERAL"       , "COLORING"      , _TL("Coloring"          ), _TL(""), CSG_String::Format("%s|%s|%s|%s", _TL("Classified"), _TL("Discrete Colors"), _TL("Graduated Colors"), _TL("RGB Coded Values")), Coloring);
+	m_Parameters.Add_Choice    ("COLORING"      , "COLORS_ATTR"   , _TL("Attribute"         ), _TL(""), Attributes, Attribute);
+	m_Parameters.Add_FixedTable("COLORING"      , "COLORS_LUT"    , _TL("Lookup Table"      ), _TL(""));
+	m_Parameters.Add_Colors    ("COLORING"      , "COLORS"        , _TL("Colors"            ), _TL(""));
+	m_Parameters.Add_Choice    ("COLORING"      , "COLORS_FIT"    , _TL("Color Stretch"     ), _TL(""), CSG_String::Format("%s|%s", _TL("constant range"), _TL("zoomed extent")), 1);
+	m_Parameters.Add_Range     ("COLORS_FIT"    , "COLORS_RANGE"  , _TL("Stretch Range"     ), _TL(""));
+	m_Parameters.Add_Choice    ("COLORS_FIT"    , "COLORS_STRETCH", _TL("Adjustment"        ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("Linear"), _TL("Standard Deviation"), _TL("Percent Clip")), 2);
+	m_Parameters.Add_Range     ("COLORS_STRETCH", "COLORS_LINEAR" , _TL("Linear"            ), _TL(""), 5., 95., 0., true, 100., true);
+	m_Parameters.Add_Double    ("COLORS_STRETCH", "COLORS_STDDEV" , _TL("Standard Deviation"), _TL(""), 2., 0.1, true);
+	m_Parameters.Add_Bool      ("COLORS_STDDEV" , "COLORS_INRANGE", _TL("Keep in Range"     ), _TL(""), false);
+	m_Parameters.Add_Range     ("COLORS_STRETCH", "COLORS_PERCENT", _TL("Percent Clip"      ), _TL(""), 2., 98., 0., true, 100., true);
 
-	m_Parameters.Add_Bool      ("GENERAL"    , "DIM"          , _TL("Dim"               ), _TL(""), false);
-	m_Parameters.Add_Range     ("DIM"        , "DIM_RANGE"    , _TL("Distance Range"    ), _TL(""), 1., 2., 0., true);
+	m_Parameters.Add_Bool      ("GENERAL"       , "DIM"           , _TL("Dim"               ), _TL(""), false);
+	m_Parameters.Add_Range     ("DIM"           , "DIM_RANGE"     , _TL("Distance Range"    ), _TL(""), 1., 2., 0., true);
 
-	m_Parameters.Add_Int       ("GENERAL"    , "SIZE"         , _TL("Size"              ), _TL(""), 1, 1, true);
-	m_Parameters.Add_Double    ("GENERAL"    , "SIZE_SCALE"   , _TL("Size Scaling"      ), _TL(""), 0., 0., true);
+	m_Parameters.Add_Int       ("GENERAL"       , "SIZE"          , _TL("Size"              ), _TL(""), 1, 1, true);
+	m_Parameters.Add_Double    ("GENERAL"       , "SIZE_SCALE"    , _TL("Size Scaling"      ), _TL(""), 0., 0., true);
 
-	m_Parameters.Add_Choice    ("GENERAL"    , "OVERVIEW_ATTR", _TL("Overview Content"  ), _TL(""), CSG_String::Format("%s|%s", _TL("average value"), _TL("number of points")), 0);
+	m_Parameters.Add_Choice    ("GENERAL"       , "OVERVIEW_ATTR" , _TL("Overview Content"  ), _TL(""), CSG_String::Format("%s|%s", _TL("average value"), _TL("number of points")), 0);
 
 	//-----------------------------------------------------
 	CSG_Table &LUT = *m_Parameters("COLORS_LUT")->asTable(); LUT.Del_Records();
@@ -179,6 +184,11 @@ C3D_Viewer_PointCloud_Panel::C3D_Viewer_PointCloud_Panel(wxWindow *pParent, CSG_
 
 	//-----------------------------------------------------
 	m_Extent = pPoints->Get_Extent();
+	
+	if( Extent.Get_Area() > 0. && Extent.Intersects(pPoints->Get_Extent()) )
+	{
+		m_Extent.Intersect(Extent);
+	}
 
 	m_Selection.Create(sizeof(sLong), 0, TSG_Array_Growth::SG_ARRAY_GROWTH_2);
 
@@ -195,10 +205,13 @@ int C3D_Viewer_PointCloud_Panel::On_Parameter_Changed(CSG_Parameters *pParameter
 {
 	if( pParameter->Cmp_Identifier("COLORS_ATTR") )
 	{
-		double m = m_pPoints->Get_Mean(pParameter->asInt()), s = m_pPoints->Get_StdDev(pParameter->asInt()) * (*pParameters)("COLORS_STDDEV")->asDouble();
+		if( (*pParameters)("COLORS_FIT")->asInt() == 0 )
+		{
+			int Field = (*pParameters)("COLORS_ATTR")->asInt();
 
-		pParameters->Set_Parameter("COLORS_RANGE.MIN", m - s);
-		pParameters->Set_Parameter("COLORS_RANGE.MAX", m + s);
+			pParameters->Set_Parameter("COLORS_RANGE.MIN", m_pPoints->Get_Histogram(Field).Get_Percentile(m_Parameters("COLORS_PERCENT.MIN")->asDouble()));
+			pParameters->Set_Parameter("COLORS_RANGE.MAX", m_pPoints->Get_Histogram(Field).Get_Percentile(m_Parameters("COLORS_PERCENT.MAX")->asDouble()));
+		}
 	}
 
 	return( CSG_3DView_Panel::On_Parameter_Changed(pParameters, pParameter) );
@@ -216,7 +229,15 @@ int C3D_Viewer_PointCloud_Panel::On_Parameters_Enable(CSG_Parameters *pParameter
 
 	if( pParameter->Cmp_Identifier("COLORS_FIT") )
 	{
-		pParameters->Set_Enabled("COLORS_RANGE", pParameter->asInt() == 1);
+		pParameters->Set_Enabled("COLORS_RANGE"  , pParameter->asInt() == 0);
+		pParameters->Set_Enabled("COLORS_STRETCH", pParameter->asInt() == 1);
+	}
+
+	if( pParameter->Cmp_Identifier("COLORS_STRETCH") )
+	{
+		pParameters->Set_Enabled("COLORS_LINEAR" , pParameter->asInt() == 0);
+		pParameters->Set_Enabled("COLORS_STDDEV" , pParameter->asInt() == 1);
+		pParameters->Set_Enabled("COLORS_PERCENT", pParameter->asInt() == 2);
 	}
 
 	if( pParameter->Cmp_Identifier("DIM") )
@@ -263,49 +284,98 @@ void C3D_Viewer_PointCloud_Panel::Update_Statistics(void)
 
 	m_Selection.Set_Array(0);
 
-	int    cField = m_Parameters("COLORS_ATTR"  )->asInt   ();
-	double cSigma = m_Parameters("COLORS_STDDEV")->asDouble();
+	int Field = m_Parameters["COLORS_ATTR"].asInt();
 
 	if( m_Extent.is_Equal(m_pPoints->Get_Extent()) )
 	{
-		if( m_Parameters("COLORS_FIT")->asInt() == 0 )
-		{
-			m_Parameters("COLORS_RANGE")->asRange()->Set_Range(
-				m_pPoints->Get_Mean(cField) - cSigma * m_pPoints->Get_StdDev(cField),
-				m_pPoints->Get_Mean(cField) + cSigma * m_pPoints->Get_StdDev(cField)
-			);
-		}
+		m_Data_Min.z = m_pPoints->Get_Minimum(2); // Get_ZMin(); ToDo in CSG_PointCloud class!!!
+		m_Data_Max.z = m_pPoints->Get_Maximum(2); // Get_ZMax(); ToDo in CSG_PointCloud class!!!
 
-		m_Data_Min.z = m_pPoints->Get_Minimum(2);	// Get_ZMin();	ToDo in CSG_PointCloud class!!!
-		m_Data_Max.z = m_pPoints->Get_Maximum(2);	// Get_ZMax();	ToDo in CSG_PointCloud class!!!
+		if( m_Parameters["COLORS_FIT"].asInt() == 1 )
+		{
+			double Minimum = m_pPoints->Get_Minimum(Field), Maximum = m_pPoints->Get_Maximum(Field);
+
+			switch( m_Parameters["COLORS_STRETCH"].asInt() )
+			{
+			case  0: // linear
+				Minimum = m_pPoints->Get_Minimum(Field) + m_pPoints->Get_Range (Field) * m_Parameters["COLORS_LINEAR.MIN"].asDouble() / 100.;
+				Maximum = m_pPoints->Get_Minimum(Field) + m_pPoints->Get_Range (Field) * m_Parameters["COLORS_LINEAR.MAX"].asDouble() / 100.;
+				break;
+
+			case  1: // standard deviation
+				Minimum = m_pPoints->Get_Mean(Field) - m_pPoints->Get_StdDev(Field) * m_Parameters["COLORS_STDDEV"].asDouble(); if( m_Parameters["COLORS_INRANGE"].asBool() && Minimum < m_pPoints->Get_Minimum(Field) ) { Minimum = m_pPoints->Get_Minimum(Field); }
+				Maximum = m_pPoints->Get_Mean(Field) + m_pPoints->Get_StdDev(Field) * m_Parameters["COLORS_STDDEV"].asDouble(); if( m_Parameters["COLORS_INRANGE"].asBool() && Maximum > m_pPoints->Get_Maximum(Field) ) { Maximum = m_pPoints->Get_Maximum(Field); }
+				break;
+
+			default: // percent clip
+				Minimum = m_pPoints->Get_Histogram(Field).Get_Percentile(m_Parameters["COLORS_PERCENT.MIN"].asDouble());
+				Maximum = m_pPoints->Get_Histogram(Field).Get_Percentile(m_Parameters["COLORS_PERCENT.MAX"].asDouble());
+				break;
+			}
+
+			m_Parameters["COLORS_RANGE.MIN"].Set_Value(Minimum);
+			m_Parameters["COLORS_RANGE.MAX"].Set_Value(Maximum);
+		}
 	}
 	else
 	{
-		CSG_Simple_Statistics zStats, cStats;
+		CSG_Simple_Statistics Statistics[2]; bool bColors = m_Parameters["COLORS_FIT"].asInt() == 1;
 
 		for(sLong i=0; i<m_pPoints->Get_Count(); i++)
 		{
-			m_pPoints->Set_Cursor(i);
-
-			if( m_Extent.Contains(m_pPoints->Get_X(), m_pPoints->Get_Y()) && m_Selection.Inc_Array() )
+			if( m_Extent.Contains(m_pPoints->Get_X(i), m_pPoints->Get_Y(i)) && m_Selection.Inc_Array() )
 			{
-				*((sLong *)m_Selection.Get_Entry(m_Selection.Get_Size() - 1))	= i;
+				*((sLong *)m_Selection.Get_Entry(m_Selection.Get_Size() - 1)) = i;
 
-				cStats += m_pPoints->Get_Value(cField);
-				zStats += m_pPoints->Get_Z();
+				Statistics[0] += m_pPoints->Get_Z(i);
+
+				if( bColors )
+				{
+					Statistics[1] += m_pPoints->Get_Value(i, Field);
+				}
 			}
 		}
 
-		if( m_Parameters("COLORS_FIT")->asInt() == 0 )
-		{
-			m_Parameters("COLORS_RANGE")->asRange()->Set_Range(
-				cStats.Get_Mean() - cSigma * cStats.Get_StdDev(),
-				cStats.Get_Mean() + cSigma * cStats.Get_StdDev()
-			);
-		}
+		m_Data_Min.z = Statistics[0].Get_Minimum();
+		m_Data_Max.z = Statistics[0].Get_Maximum();
 
-		m_Data_Min.z = zStats.Get_Minimum();
-		m_Data_Max.z = zStats.Get_Maximum();
+		if( bColors && m_Selection.Get_Size() > 0 )
+		{
+			double Minimum = Statistics[1].Get_Minimum(), Maximum = Statistics[1].Get_Maximum();
+
+			switch( m_Parameters["COLORS_STRETCH"].asInt() )
+			{
+			case  0: // linear
+				Minimum = Statistics[1].Get_Minimum() + Statistics[1].Get_Range() * m_Parameters["COLORS_LINEAR.MIN"].asDouble() / 100.;
+				Maximum = Statistics[1].Get_Minimum() + Statistics[1].Get_Range() * m_Parameters["COLORS_LINEAR.MAX"].asDouble() / 100.;
+				break;
+
+			case  1: // standard deviation
+				Minimum = Statistics[1].Get_Mean() - Statistics[1].Get_StdDev() * m_Parameters["COLORS_STDDEV"].asDouble(); if( m_Parameters["COLORS_INRANGE"].asBool() && Minimum < Statistics[1].Get_Minimum() ) { Minimum = Statistics[1].Get_Minimum(); }
+				Maximum = Statistics[1].Get_Mean() + Statistics[1].Get_StdDev() * m_Parameters["COLORS_STDDEV"].asDouble(); if( m_Parameters["COLORS_INRANGE"].asBool() && Maximum > Statistics[1].Get_Maximum() ) { Maximum = Statistics[1].Get_Maximum(); }
+				break;
+
+			default: // percent clip
+				{
+					CSG_Histogram Histogram(64, Statistics[1].Get_Minimum(), Statistics[1].Get_Maximum());
+
+					for(sLong i=0; i<m_Selection.Get_Size(); i++)
+					{
+						Histogram += m_pPoints->Get_Value(*((sLong *)m_Selection.Get_Entry(i)), Field);
+					}
+
+					if( Histogram.Update() )
+					{
+						Minimum = Histogram.Get_Percentile(m_Parameters["COLORS_PERCENT.MIN"].asDouble());
+						Maximum = Histogram.Get_Percentile(m_Parameters["COLORS_PERCENT.MAX"].asDouble());
+					}
+				}
+				break;
+			}
+
+			m_Parameters["COLORS_RANGE.MIN"].Set_Value(Minimum);
+			m_Parameters["COLORS_RANGE.MAX"].Set_Value(Maximum);
+		}
 	}
 }
 
@@ -530,7 +600,7 @@ public:
 		: wxDialog(pParent, wxID_ANY, _TL("Overview"), wxDefaultPosition, wxDefaultSize, wxCAPTION|wxCLOSE_BOX|wxSTAY_ON_TOP)
 	#endif
 	{
-		m_pPanel = pPanel;
+		m_pPanel = pPanel; m_Extent = m_pPanel->Get_Extent();
 
 		m_bCount = m_pPanel->m_Parameters("OVERVIEW_ATTR")->asInt() == 1;
 
@@ -607,7 +677,7 @@ private:
 
 	wxPoint						m_Mouse_Down;
 
-	CSG_Rect					m_Selection;
+	CSG_Rect					m_Extent;
 
 	wxImage						m_Image;
 
@@ -645,16 +715,16 @@ private:
 		{
 			Draw_Inverse(m_Mouse_Down, event.GetPosition());
 
-			m_pPanel->Set_Extent(m_Selection);
+			m_pPanel->Set_Extent(m_Extent);
 		}
-		else if( m_Selection.Get_XRange() > 0. && m_Selection.Get_YRange() > 0. )
+		else if( m_Extent.Get_XRange() > 0. && m_Extent.Get_YRange() > 0. )
 		{
-			double dx = (m_Count.Get_XMin() + event.GetX() * m_Count.Get_XRange() / GetClientSize().GetWidth ()) - m_Selection.Get_XCenter();
-			double dy = (m_Count.Get_YMax() - event.GetY() * m_Count.Get_YRange() / GetClientSize().GetHeight()) - m_Selection.Get_YCenter();
+			double dx = (m_Count.Get_XMin() + event.GetX() * m_Count.Get_XRange() / GetClientSize().GetWidth ()) - m_Extent.Get_XCenter();
+			double dy = (m_Count.Get_YMax() - event.GetY() * m_Count.Get_YRange() / GetClientSize().GetHeight()) - m_Extent.Get_YCenter();
 
-			m_Selection.Move(dx, dy);
+			m_Extent.Move(dx, dy);
 
-			m_pPanel->Set_Extent(m_Selection);
+			m_pPanel->Set_Extent(m_Extent);
 		}
 
 		Refresh(false); ((CSG_3DView_Dialog *)m_pPanel->GetParent())->Update_Controls();
@@ -669,9 +739,9 @@ private:
 	//---------------------------------------------------------
 	void						On_Mouse_RDown	(wxMouseEvent &event)
 	{
-		m_Selection.Assign(0., 0., 0., 0.);
+		m_Extent.Create(0., 0., 0., 0.);
 
-		m_pPanel->Set_Extent(m_Selection);
+		m_pPanel->Set_Extent(m_Extent);
 
 		Refresh(false); ((CSG_3DView_Dialog *)m_pPanel->GetParent())->Update_Controls();
 	}
@@ -723,14 +793,14 @@ private:
 			dc.DrawBitmap(wxBitmap(m_Image), GetClientRect().GetTopLeft());
 		}
 
-		if( m_Selection.Get_XRange() > 0. && m_Selection.Get_YRange() > 0. )
+		if( m_Extent.Get_XRange() > 0. && m_Extent.Get_YRange() > 0. )
 		{
 			double d = GetClientSize().GetWidth () / m_Count.Get_XRange();
 
-			int ax = (int)(d * (m_Selection.Get_XMin() - m_Count.Get_XMin()));
-			int bx = (int)(d * (m_Selection.Get_XMax() - m_Count.Get_XMin()));
-			int ay = (int)(d * (m_Count.Get_YMax() - m_Selection.Get_YMax()));
-			int by = (int)(d * (m_Count.Get_YMax() - m_Selection.Get_YMin()));
+			int ax = (int)(d * (m_Extent.Get_XMin() - m_Count.Get_XMin()));
+			int bx = (int)(d * (m_Extent.Get_XMax() - m_Count.Get_XMin()));
+			int ay = (int)(d * (m_Count.Get_YMax() - m_Extent.Get_YMax()));
+			int by = (int)(d * (m_Count.Get_YMax() - m_Extent.Get_YMin()));
 
 			dc.SetPen(wxPen(*wxBLACK));
 			dc.DrawLine(ax, ay, ax, by); dc.DrawLine(ax, by, bx, by);
@@ -749,7 +819,7 @@ private:
 		{
 			double d = m_Count.Get_XRange() / GetClientSize().GetWidth();
 
-			m_Selection.Assign(
+			m_Extent.Create(
 				m_Count.Get_XMin() + d * A.x, m_Count.Get_YMax() - d * A.y,
 				m_Count.Get_XMin() + d * B.x, m_Count.Get_YMax() - d * B.y
 			);
@@ -833,10 +903,10 @@ END_EVENT_TABLE()
 class C3D_Viewer_PointCloud_Dialog : public CSG_3DView_Dialog
 {
 public:
-	C3D_Viewer_PointCloud_Dialog(CSG_PointCloud *pPoints, int Coloring, int Attribute)
+	C3D_Viewer_PointCloud_Dialog(CSG_PointCloud *pPoints, int Coloring, int Attribute, const CSG_Rect &Extent)
 		: CSG_3DView_Dialog(_TL("Point Cloud Viewer"))
 	{
-		Create(new C3D_Viewer_PointCloud_Panel(this, pPoints, Coloring, Attribute));
+		Create(new C3D_Viewer_PointCloud_Panel(this, pPoints, Coloring, Attribute, Extent));
 
 		//-------------------------------------------------
 		Add_Spacer();
@@ -1054,6 +1124,9 @@ C3D_Viewer_PointCloud::C3D_Viewer_PointCloud(void)
 		_TL(""),
 		PARAMETER_INPUT
 	);
+
+	Parameters.Add_Range("", "XRANGE", _TL("X Range"), _TL(""), 0., 0.)->Set_Enabled(false);
+	Parameters.Add_Range("", "YRANGE", _TL("Y Range"), _TL(""), 0., 0.)->Set_Enabled(false);
 }
 
 
@@ -1084,17 +1157,38 @@ bool C3D_Viewer_PointCloud::On_Execute(void)
 		return( false );
 	}
 
-	int Coloring, Attribute = 2; CSG_Parameter *pParameter = DataObject_Get_Parameter(pPoints, "COLORS_TYPE");
+	CSG_Parameter *pParameter = DataObject_Get_Parameter(pPoints, "COLORS_TYPE");
 
-	switch( pParameter ? pParameter->asInt() : -1 )
+	int Classifier = pParameter ? pParameter->asInt() : -1;
+
+	if( Classifier == 1 ) // Classified with Normalization not supported (yet)
 	{
-	case  1: Coloring = 0; if( (pParameter = DataObject_Get_Parameter(pPoints,    "LUT_ATTRIB")) != NULL ) { Attribute = pParameter->asInt(); } break; // Classified
-	case  2: Coloring = 1; if( (pParameter = DataObject_Get_Parameter(pPoints, "METRIC_ATTRIB")) != NULL ) { Attribute = pParameter->asInt(); } break; // Discrete Colors
-	default: Coloring = 2; if( (pParameter = DataObject_Get_Parameter(pPoints, "METRIC_ATTRIB")) != NULL ) { Attribute = pParameter->asInt(); } break; // Graduated Colors
-	case  4: Coloring = 3; if( (pParameter = DataObject_Get_Parameter(pPoints,    "RGB_ATTRIB")) != NULL ) { Attribute = pParameter->asInt(); } break; // RGB Coded Values
+		int Normal; pParameter = DataObject_Get_Parameter(pPoints, "LUT_NORMAL");
+
+		if( pParameter && pParameter->asChoice() && pParameter->asChoice()->Get_Data(Normal) && Normal >= 0 )
+		{
+			Classifier = -1;
+		}
 	}
 
-	C3D_Viewer_PointCloud_Dialog dlg(pPoints, Coloring, Attribute);
+	#define Get_Attribute(field) { CSG_Parameter *p = DataObject_Get_Parameter(pPoints, field); if( p && p->asChoice() ) { p->asChoice()->Get_Data(Attribute); } }
+
+	int Coloring, Attribute = 2;
+
+	switch( Classifier )
+	{
+	case  1: Coloring = 0; Get_Attribute(    "LUT_FIELD"); break; // Classified
+	case  2: Coloring = 1; Get_Attribute( "METRIC_FIELD"); break; // Discrete Colors
+	default: Coloring = 2; Get_Attribute( "METRIC_FIELD"); break; // Graduated Colors
+	case  4: Coloring = 3; Get_Attribute(   "RGB_ATTRIB"); break; // RGB Coded Values
+	}
+
+	CSG_Rect Extent(
+		Parameters["XRANGE.MIN"].asDouble(), Parameters["YRANGE.MIN"].asDouble(),
+		Parameters["XRANGE.MAX"].asDouble(), Parameters["YRANGE.MAX"].asDouble()
+	);
+
+	C3D_Viewer_PointCloud_Dialog dlg(pPoints, Coloring, Attribute, Extent);
 
 	switch( Coloring )
 	{
